/* Fichier "algo_hyperbole.c"
 * Algorithme dit de "l'hyperboloide  deux nappes,
 * avec volcans et mammelons"
 */

#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>

#include "terrain.h"

/* ======================================================
 * TER_gen_terrain_hyperbole
 *
 * Description: genere des hyperboloides tronques  z = r/3,
 *  o l'on ajoute suivant le cas une demi sphere superieure (mammelon)
 *  ou inferieure (volcan), ou rien du tout (plat).
 *  On fait un certain nombre d'iterations, si il y a dj quelque chose,
 *  on additionne l'ancienne hauteur avec la nouvelle.
 *
 *  NB: cet algorithme montre qu'on peut utiliser ce projet comme 
 *      traceur de surfaces regulieres definies implicitement.
 * =====================================================
 */
void
TER_gen_terrain_hyperbole (TER_terrain_t * terrain)
{
  int x1, y1, x2, y2;
  float z = 0;
  float radius = 1.0f;
  int iter = 0;
  float dist = 0.0f;
  float r1 = 0.0f;
  int epsilon = 0;
  TER_param_hyperb_t *param = (TER_param_hyperb_t *) terrain->spec_param; 

  if (!param) {
    fprintf(stderr, "Error: no parameters found\n");
    return;
  }
  
  srand (time (NULL));

  for (iter = 0; iter < param->num_iter; iter++) {

    /* on tire un point au hasard, centre de l'hyberboloide */
    x1 = random_int (terrain->size - 1);
    y1 = random_int (terrain->size - 1);
    
    /* on tire son rayon au hasard entre radius_min et radius_max */
    radius = random_float (param->radius_min, param->radius_max);
    /* pour une distance inferieure  ce rayon divis par 3, on tronque */
    r1 = radius * 0.333f;

    /* tire au hasard selon les parametres, si l'on ajoute une sphere, 
        retranche une sphere (volcan) ou si on tronque net */
    if ((param->epsilon & ALGO_HYPERB_CRATERS)
      && (param->epsilon & ALGO_HYPERB_SPHERES)) 
    epsilon = random_int(3) - 1;
    
    else if (param->epsilon & ALGO_HYPERB_CRATERS)
    epsilon = random_int (2)-1;
    
    else if (param->epsilon & ALGO_HYPERB_SPHERES)
    epsilon = random_int (2);

    else epsilon=0;
    
    /* calcule les hauteurs des points */  
    for (x2 = 0; x2 < terrain->size; x2++) {
      for (y2 = 0; y2 < terrain->size; y2++) {

	dist = sqrt ((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));

	if ((dist < radius) && (dist > r1) && (dist > 0))
	/* hyperboloide z=1/(dist^power) */
	  z =
	    param->factor /
	    (float) (pow ((double) (dist), param->power))
	    -
	    param->factor /
	    (float) (pow ((double) (radius), param->power));
	    
	/* on tronque ! */    
	else if (dist <= r1)
	  z =
	    param->factor /
	    (float) (pow ((double) (r1), param->power))
	    -
	    param->factor /
	    (float) (pow ((double) (radius), param->power))

          /* on ajoute soit sphere sup, soit sphere inf, soit rien */
	  + (float) epsilon *sqrt (r1 * r1 -
				   ((x2 - x1) * (x2 - x1) +
				    (y2 - y1) * (y2 - y1)));

	else
	z = 0;

        /* ajoute la nouvelle altitude avec l'eventuelle ancienne */
	z += TER_get_heightmap_value (terrain, x2, y2);

	if (z > terrain->max_height)
	  terrain->max_height = z;

	TER_set_heightmap_value (terrain, x2, y2, z);
      }
    }
  }

  printf ("HYPERB: max_height=%f\n", terrain->max_height);

}
